@@ -1750,6 +1750,10 @@ class VersionTest(Base):
 
 class AggregateChangesTest(Base):
 
+    def setUp(self):
+        super(AggregateChangesTest, self).setUp()
+        self.aggregator = scanner._ChangeAggregator()
+
     def test_ignore(self):
         entry = mock.Mock()
         n = self.get_note_num()
@@ -1766,7 +1770,7 @@ class AggregateChangesTest(Base):
                 )
             )
         ]
-        results = scanner._aggregate_changes(entry, changes, 'prefix')
+        results = self.aggregator.aggregate_changes(entry, changes)
         self.assertEqual(
             [],
             results,
@@ -1788,12 +1792,77 @@ class AggregateChangesTest(Base):
                 )
             )
         ]
-        results = list(scanner._aggregate_changes(entry, changes, 'prefix'))
+        results = list(self.aggregator.aggregate_changes(entry, changes))
         self.assertEqual(
             [('%016x' % n, 'add', name, 'commit-id')],
             results,
         )
 
+    def test_add_multiple_after_delete(self):
+        # Adding multiple files in one commit using the same UID but
+        # different slug after we have seen a delete for the same UID
+        # causes the files to be ignored.
+        entry = mock.Mock()
+        n = self.get_note_num()
+        uid = '%016x' % n
+        changes = []
+        for i in range(2):
+            name = 'prefix/add%d-%s.yaml' % (i, uid)
+            entry.commit.id = 'commit-id'
+            changes.append(
+                diff_tree.TreeChange(
+                    type=diff_tree.CHANGE_ADD,
+                    old=objects.TreeEntry(path=None, mode=None, sha=None),
+                    new=objects.TreeEntry(
+                        path=name.encode('utf-8'),
+                        mode='0222',
+                        sha='not-a-hash',
+                    )
+                )
+            )
+        # Set up the aggregator as though it had already seen a delete
+        # operation. Since the scan happens in reverse chronological
+        # order, the delete would have happened after the add, and we
+        # can ignore the files because the error has been corrected in
+        # a later patch.
+        self.aggregator._deleted_bad_uids.add(uid)
+        results = list(self.aggregator.aggregate_changes(entry, changes))
+        self.assertEqual([], results)
+
+    def test_add_multiple_without_delete(self):
+        # Adding multiple files in one commit using the same UID but
+        # different slug without a delete operation causes an
+        # exception.
+        entry = mock.Mock()
+        n = self.get_note_num()
+        uid = '%016x' % n
+        changes = []
+        for i in range(2):
+            name = 'prefix/add%d-%s.yaml' % (i, uid)
+            entry.commit.id = 'commit-id'
+            changes.append(
+                diff_tree.TreeChange(
+                    type=diff_tree.CHANGE_ADD,
+                    old=objects.TreeEntry(path=None, mode=None, sha=None),
+                    new=objects.TreeEntry(
+                        path=name.encode('utf-8'),
+                        mode='0222',
+                        sha='not-a-hash',
+                    )
+                )
+            )
+
+        # aggregate_changes() is a generator, so we have to wrap it in
+        # list() to process the data, so we need a little temporary
+        # function to do that and pass to assertRaises().
+        def get_results():
+            return list(self.aggregator.aggregate_changes(entry, changes))
+
+        self.assertRaises(
+            ValueError,
+            get_results,
+        )
+
     def test_delete(self):
         entry = mock.Mock()
         n = self.get_note_num()
@@ -1810,12 +1879,37 @@ class AggregateChangesTest(Base):
                 new=objects.TreeEntry(path=None, mode=None, sha=None)
             )
         ]
-        results = list(scanner._aggregate_changes(entry, changes, 'prefix'))
+        results = list(self.aggregator.aggregate_changes(entry, changes))
         self.assertEqual(
             [('%016x' % n, 'delete', name)],
             results,
         )
 
+    def test_delete_multiple(self):
+        # Delete multiple files in one commit using the same UID but
+        # different slug.
+        entry = mock.Mock()
+        n = self.get_note_num()
+        changes = []
+        expected = []
+        for i in range(2):
+            name = 'prefix/delete%d-%016x.yaml' % (i, n)
+            entry.commit.id = 'commit-id'
+            changes.append(
+                diff_tree.TreeChange(
+                    type=diff_tree.CHANGE_DELETE,
+                    old=objects.TreeEntry(
+                        path=name.encode('utf-8'),
+                        mode='0222',
+                        sha='not-a-hash',
+                    ),
+                    new=objects.TreeEntry(path=None, mode=None, sha=None),
+                )
+            )
+            expected.append(('%016x' % n, 'delete', name, 'commit-id'))
+        results = list(self.aggregator.aggregate_changes(entry, changes))
+        self.assertEqual(expected, results)
+
     def test_change(self):
         entry = mock.Mock()
         n = self.get_note_num()
@@ -1836,7 +1930,7 @@ class AggregateChangesTest(Base):
                 ),
             )
         ]
-        results = list(scanner._aggregate_changes(entry, changes, 'prefix'))
+        results = list(self.aggregator.aggregate_changes(entry, changes))
         self.assertEqual(
             [('%016x' % n, 'modify', name, 'commit-id')],
             results,
@@ -1868,7 +1962,7 @@ class AggregateChangesTest(Base):
                 new=objects.TreeEntry(path=None, mode=None, sha=None)
             )
         ]
-        results = list(scanner._aggregate_changes(entry, changes, 'prefix'))
+        results = list(self.aggregator.aggregate_changes(entry, changes))
         self.assertEqual(
             [('%016x' % n, 'rename', old_name, new_name, 'commit-id')],
             results,
@@ -1900,7 +1994,7 @@ class AggregateChangesTest(Base):
                 )
             ),
         ]
-        results = list(scanner._aggregate_changes(entry, changes, 'prefix'))
+        results = list(self.aggregator.aggregate_changes(entry, changes))
         self.assertEqual(
             [('%016x' % n, 'rename', old_name, new_name, 'commit-id')],
             results,
@@ -1946,7 +2040,7 @@ class AggregateChangesTest(Base):
                 ),
             ),
         ]]
-        results = list(scanner._aggregate_changes(entry, changes, 'prefix'))
+        results = list(self.aggregator.aggregate_changes(entry, changes))
         self.assertEqual(
             [('%016x' % n, 'modify', old_name, 'commit-id'),
              ('%016x' % n, 'modify', old_name, 'commit-id')],
